/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */

package net.craftforge.essential.supply;

import net.craftforge.essential.controller.Configuration;
import net.craftforge.essential.controller.ControllerException;
import net.craftforge.essential.controller.annotations.Produces;
import net.craftforge.essential.controller.utils.AnnotationUtils;

import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Provides an implementation of the Controllable interface in
 * a view setup.
 *
 * @author Christian Bick
 * @since 02.12.2010
 */
@SuppressWarnings("unused")
public class Producer {

    /**
     * The configuration
     */
    protected Configuration config;

    /**
     * Constructs the producer by supplying the controller configuration.
     *
     * @param config The configuration
     */
    public Producer(Configuration config) {
        this.config = config;    
    }

    /**
     * Serializes the given object to the most suitable media type in the enumeration
     * of accepted media types nd writes the serialization results using the specified writer.
     * The producer method is chosen accordingly to the best fitting accepted
     * media type. The responsible producer method must contain this media type
     * in its @Produces annotation.
     *
     * @param object The object to serialize
     * @param writer The writer
     * @param accepted The comma separated enumeration of accepted media types
     * @throws ControllerException if the production process fails
     */
    public void produce(Object object, Writer writer, String accepted) throws ControllerException {
        Method producerMethod = AnnotationUtils.getProducerMethod(this, accepted);
        try {
            producerMethod.invoke(this, object, writer);
        } catch (InvocationTargetException e) {
            Throwable cause = e.getCause();
            if (cause instanceof ControllerException) {
                throw (ControllerException)cause;
            }
            throw new ControllerException("The producer method failed during execution: " + producerMethod, cause);
        } catch (IllegalAccessException e) {
            throw new ControllerException("The producer method could not be invoked: " + producerMethod, e);
        }
    }

    /**
     * Gets the media type best matching the comma separated string of
     * accepted media types.
     *
     * @param accepted The comma separated string of accepted media types
     * @return The best matching accepted media type
     * @throws ControllerException if the producer does not supply any of the accepted media types
     */
    public String getBestMatchingMediaType(String accepted) throws ControllerException {
        return AnnotationUtils.getBestMatchingMediaType(this, Produces.class, accepted);
    }

    /**
     * Serializes the given object to plain text using the specified writer.
     *
     * @param object The object
     * @param writer The writer
     * @throws IOException Failed to initialize the writer for the output stream
     */
    @Produces({"text/plain"})
    public void plain(Object object, Writer writer) throws IOException {
        String text = object.toString();
        writer.write(text);
        writer.close();
    }

}
